@@ -150,6 +150,8 @@
 #define NO_ADDRESS_SAFETY_ANALYSIS(x) __attribute__ ((__no_address_safety_analysis__)) x
 #define WARN_UNUSED_RESULT(x) __attribute__ ((__warn_unused_result__)) x
 #define MAYBE_UNUSED(x) __attribute__ ((__unused__)) x
+#define ERRORFUNC(mesg,x) __attribute__ ((__error__ mesg)) x
+#define WARNINGFUNC(mesg,x) __attribute__ ((__warning__ mesg)) x
 #define WEAK(x) __attribute__ ((__weak__)) x
 #define HAVE_FUNC_WEAK 1
 #define RUBY_CXX_DEPRECATED(msg) __attribute__((__deprecated__(msg)))
@@ -252,7 +254,6 @@
 #define HAVE_VFORK 1
 #define HAVE_WORKING_VFORK 1
 #define HAVE_WORKING_FORK 1
-#define HAVE___SYSCALL 1
 #define HAVE__LONGJMP 1
 #define HAVE_ARC4RANDOM_BUF 1
 #define HAVE_ATAN2L 1
@@ -346,7 +347,6 @@
 #define HAVE_SIGPROCMASK 1
 #define HAVE_SINH 1
 #define HAVE_SYMLINK 1
-#define HAVE_SYSCALL 1
 #define HAVE_SYSCONF 1
 #define HAVE_TANH 1
 #define HAVE_TELLDIR 1
