@@ -2,24 +2,26 @@
 #
 #   help.rb - helper using ri
 #   	$Release Version: 0.9.6$
-#   	$Revision: 65506 $
+#   	$Revision$
 #
 # --
 #
 #
 #
 
-require 'rdoc/ri/driver'
-
 require_relative "nop"
 
 # :stopdoc:
 module IRB
   module ExtendCommand
     class Help < Nop
-      begin
-        Ri = RDoc::RI::Driver.new
-      rescue SystemExit
+      def execute(*names)
+        require 'rdoc/ri/driver'
+        opts = RDoc::RI::Driver.process_args([])
+        IRB::ExtendCommand::Help.const_set(:Ri, RDoc::RI::Driver.new(opts))
+      rescue LoadError, SystemExit
+        IRB::ExtendCommand::Help.remove_method(:execute)
+        # raise NoMethodError in ensure
       else
         def execute(*names)
           if names.empty?
@@ -35,6 +37,9 @@ module IRB
           end
           nil
         end
+        nil
+      ensure
+        execute(*names)
       end
     end
   end
