@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("3.1.") or
-    raise "ruby lib version (3.1.3) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (3.1.4) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.1/x86_64-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
   CONFIG["MINOR"] = "1"
-  CONFIG["TEENY"] = "3"
-  CONFIG["PATCHLEVEL"] = "185"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.1.3/bin/install -c'
+  CONFIG["TEENY"] = "4"
+  CONFIG["PATCHLEVEL"] = "223"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.1.4/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)31"
@@ -45,7 +45,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=31' '--with-soname=ruby31' '--with-ruby-version=3.1' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe -g' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe -g'"
+  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=31' '--with-soname=ruby31' '--with-ruby-version=3.1' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--with-compress-debug-sections=no' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe -g' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe -g'"
   CONFIG["CONFIGURE"] = "configure"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
@@ -98,7 +98,7 @@ module RbConfig
   CONFIG["strict_warnflags"] = "-std=gnu99"
   CONFIG["warnflags"] = "-Wall -Wextra -Wdeprecated-declarations -Wdivision-by-zero -Wimplicit-function-declaration -Wimplicit-int -Wmisleading-indentation -Wpointer-arith -Wshorten-64-to-32 -Wwrite-strings -Wold-style-definition -Wmissing-noreturn -Wno-cast-function-type -Wno-constant-logical-operand -Wno-long-long -Wno-missing-field-initializers -Wno-overlength-strings -Wno-parentheses-equality -Wno-self-assign -Wno-tautological-compare -Wno-unused-parameter -Wno-unused-value -Wunused-variable -Wextra-tokens -Wundef"
   CONFIG["debugflags"] = "-ggdb3"
-  CONFIG["optflags"] = "-O3 -fno-fast-math"
+  CONFIG["optflags"] = "-fno-fast-math"
   CONFIG["NULLCMD"] = ":"
   CONFIG["ENABLE_DEBUG_ENV"] = ""
   CONFIG["DLNOBJ"] = "dln.o"
@@ -151,9 +151,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.1.3/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.1.3/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.1.3/bin/install -c  -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.1.4/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.1.4/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.1.4/bin/install -c  -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
@@ -161,7 +161,7 @@ module RbConfig
   CONFIG["ASFLAGS"] = ""
   CONFIG["ARFLAGS"] = "rcD "
   CONFIG["try_header"] = ""
-  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 13.0.0\nTarget: amd64-unknown-openbsd7.3\nThread model: posix\nInstalledDir: /usr/bin"
+  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 16.0.6\nTarget: amd64-unknown-openbsd7.5\nThread model: posix\nInstalledDir: /usr/bin"
   CONFIG["CC_VERSION"] = "$(CC) --version"
   CONFIG["MJIT_CC"] = "/usr/bin/cc"
   CONFIG["CSRCFLAG"] = ""
@@ -186,18 +186,18 @@ module RbConfig
   CONFIG["AS"] = "as"
   CONFIG["AR"] = "ar"
   CONFIG["CC"] = "cc"
-  CONFIG["target_os"] = "openbsd7.3"
+  CONFIG["target_os"] = "openbsd7.5"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "x86_64"
-  CONFIG["target"] = "x86_64-unknown-openbsd7.3"
-  CONFIG["host_os"] = "openbsd7.3"
+  CONFIG["target"] = "x86_64-unknown-openbsd7.5"
+  CONFIG["host_os"] = "openbsd7.5"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "x86_64"
-  CONFIG["host"] = "x86_64-unknown-openbsd7.3"
-  CONFIG["build_os"] = "openbsd7.3"
+  CONFIG["host"] = "x86_64-unknown-openbsd7.5"
+  CONFIG["build_os"] = "openbsd7.5"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "x86_64"
-  CONFIG["build"] = "x86_64-unknown-openbsd7.3"
+  CONFIG["build"] = "x86_64-unknown-openbsd7.5"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
