@@ -7,7 +7,7 @@ return if ENV['RUBY_DEBUG_ENABLE'] == '0'
 if $0.end_with?('bin/bundle') && ARGV.first == 'exec'
   trace_var(:$0) do |file|
     trace_var(:$0, nil)
-    if /-r (#{__dir__}\S+)/ =~ ENV['RUBYOPT']
+    if /-r (#{Regexp.escape(__dir__)}\S+)/ =~ ENV['RUBYOPT']
       lib = $1
       $LOADED_FEATURES.delete_if{|path| path.start_with?(__dir__)}
       ENV['RUBY_DEBUG_INITIAL_SUSPEND_PATH'] = file
@@ -19,6 +19,15 @@ if $0.end_with?('bin/bundle') && ARGV.first == 'exec'
   return
 end
 
+# restore RUBYOPT
+if (added_opt = ENV['RUBY_DEBUG_ADDED_RUBYOPT']) &&
+   (rubyopt = ENV['RUBYOPT']) &&
+   rubyopt.end_with?(added_opt)
+
+  ENV['RUBYOPT'] = rubyopt.delete_suffix(added_opt)
+  ENV['RUBY_DEBUG_ADDED_RUBYOPT'] = nil
+end
+
 require_relative 'frame_info'
 require_relative 'config'
 require_relative 'thread_client'
@@ -31,7 +40,8 @@ $LOADED_FEATURES << 'debug.rb'
 $LOADED_FEATURES << File.expand_path(File.join(__dir__, '..', 'debug.rb'))
 require 'debug' # invalidate the $LOADED_FEATURE cache
 
-require 'json' if ENV['RUBY_DEBUG_TEST_MODE']
+require 'json' if ENV['RUBY_DEBUG_TEST_UI'] == 'terminal'
+require 'pp'
 
 class RubyVM::InstructionSequence
   def traceable_lines_norec lines
@@ -56,23 +66,22 @@ class RubyVM::InstructionSequence
 
   def type
     self.to_a[9]
-  end
+  end unless method_defined?(:type)
 
-  def argc
-    self.to_a[4][:arg_size]
-  end
-
-  def locals
-    self.to_a[10]
-  end
+  def parameters_symbols
+    ary = self.to_a
+    argc = ary[4][:arg_size]
+    locals = ary.to_a[10]
+    locals[0...argc]
+  end unless method_defined?(:parameters_symbols)
 
   def last_line
     self.to_a[4][:code_location][2]
-  end
+  end unless method_defined?(:last_line)
 
   def first_line
     self.to_a[4][:code_location][0]
-  end
+  end unless method_defined?(:first_line)
 end
 
 module DEBUGGER__
@@ -82,8 +91,10 @@ module DEBUGGER__
   class Session
     attr_reader :intercepted_sigint_cmd, :process_group
 
-    def initialize ui
-      @ui = ui
+    include Color
+
+    def initialize
+      @ui = nil
       @sr = SourceRepository.new
       @bps = {} # bp.key => bp
                 #   [file, line] => LineBreakpoint
@@ -104,13 +115,13 @@ module DEBUGGER__
       @intercept_trap_sigint = false
       @intercepted_sigint_cmd = 'DEFAULT'
       @process_group = ProcessGroup.new
-      @subsession = nil
+      @subsession_stack = []
 
       @frame_map = {} # for DAP: {id => [threadId, frame_depth]} and CDP: {id => frame_depth}
       @var_map   = {1 => [:globals], } # {id => ...} for DAP
       @src_map   = {} # {id => src}
 
-      @script_paths = [File.absolute_path($0)] # for CDP
+      @scr_id_map = {} # for CDP
       @obj_map = {} # { object_id => ... } for CDP
 
       @tp_thread_begin = nil
@@ -120,9 +131,6 @@ module DEBUGGER__
       @tp_load_script.enable
 
       @thread_stopper = thread_stopper
-
-      activate
-
       self.postmortem = CONFIG[:postmortem]
     end
 
@@ -134,15 +142,13 @@ module DEBUGGER__
       @bps.has_key? [file, line]
     end
 
-    def activate on_fork: false
+    def activate ui = nil, on_fork: false
+      @ui = ui if ui
+
       @tp_thread_begin&.disable
       @tp_thread_begin = nil
 
-      if on_fork
-        @ui.activate self, on_fork: true
-      else
-        @ui.activate self, on_fork: false
-      end
+      @ui.activate self, on_fork: on_fork
 
       q = Queue.new
       @session_server = Thread.new do
@@ -201,34 +207,44 @@ module DEBUGGER__
       deactivate
     end
 
+    def request_tc(req)
+      @tc << req
+    end
+
     def process_event evt
       # variable `@internal_info` is only used for test
       tc, output, ev, @internal_info, *ev_args = evt
+
       output.each{|str| @ui.puts str} if ev != :suspend
 
-      case ev
-
-      when :thread_begin # special event, tc is nil
+      # special event, tc is nil
+      # and we don't want to set @tc to the newly created thread's ThreadClient
+      if ev == :thread_begin
         th = ev_args.shift
         q = ev_args.shift
         on_thread_begin th
         q << true
 
-      when :init
-        wait_command_loop tc
+        return
+      end
 
+      @tc = tc
+
+      case ev
+      when :init
+        enter_subsession
+        wait_command_loop
       when :load
         iseq, src = ev_args
         on_load iseq, src
-        @ui.event :load
-        tc << :continue
+        request_tc :continue
 
       when :trace
         trace_id, msg = ev_args
         if t = @tracers.values.find{|t| t.object_id == trace_id}
           t.puts msg
         end
-        tc << :continue
+        request_tc :continue
 
       when :suspend
         enter_subsession if ev_args.first != :replay
@@ -238,26 +254,25 @@ module DEBUGGER__
         when :breakpoint
           bp, i = bp_index ev_args[1]
           clean_bps unless bp
-          @ui.event :suspend_bp, i, bp, tc.id
+          @ui.event :suspend_bp, i, bp, @tc.id
         when :trap
-          @ui.event :suspend_trap, sig = ev_args[1], tc.id
+          @ui.event :suspend_trap, sig = ev_args[1], @tc.id
 
           if sig == :SIGINT && (@intercepted_sigint_cmd.kind_of?(Proc) || @intercepted_sigint_cmd.kind_of?(String))
             @ui.puts "#{@intercepted_sigint_cmd.inspect} is registered as SIGINT handler."
             @ui.puts "`sigint` command execute it."
           end
         else
-          @ui.event :suspended, tc.id
+          @ui.event :suspended, @tc.id
         end
 
         if @displays.empty?
-          wait_command_loop tc
+          wait_command_loop
         else
-          tc << [:eval, :display, @displays]
+          request_tc [:eval, :display, @displays]
         end
-
       when :result
-        raise "[BUG] not in subsession" unless @subsession
+        raise "[BUG] not in subsession" if @subsession_stack.empty?
 
         case ev_args.first
         when :try_display
@@ -286,14 +301,14 @@ module DEBUGGER__
           # ignore
         end
 
-        wait_command_loop tc
+        wait_command_loop
 
       when :dap_result
         dap_event ev_args # server.rb
-        wait_command_loop tc
+        wait_command_loop
       when :cdp_result
         cdp_event ev_args
-        wait_command_loop tc
+        wait_command_loop
       end
     end
 
@@ -326,9 +341,7 @@ module DEBUGGER__
       "DEBUGGER__::SESSION"
     end
 
-    def wait_command_loop tc
-      @tc = tc
-
+    def wait_command_loop
       loop do
         case wait_command
         when :retry
@@ -369,7 +382,7 @@ module DEBUGGER__
           @ui.puts "(rdbg:#{@preset_command.source}) #{line}"
         end
       else
-        @ui.puts "INTERNAL_INFO: #{JSON.generate(@internal_info)}" if ENV['RUBY_DEBUG_TEST_MODE']
+        @ui.puts "INTERNAL_INFO: #{JSON.generate(@internal_info)}" if ENV['RUBY_DEBUG_TEST_UI'] == 'terminal'
         line = @ui.readline prompt
       end
 
@@ -510,8 +523,8 @@ module DEBUGGER__
       #   * break and run `<command>` before stopping.
       # * `b[reak] ... do: <command>`
       #   * break and run `<command>`, and continue.
-      # * `b[reak] ... path: <path_regexp>`
-      #   * break if the triggering event's path matches <path_regexp>.
+      # * `b[reak] ... path: <path>`
+      #   * break if the path matches to `<path>`. `<path>` can be a regexp with `/regexp/`.
       # * `b[reak] if: <expr>`
       #   * break if: `<expr>` is true at any lines.
       #   * Note that this feature is super slow.
@@ -532,32 +545,6 @@ module DEBUGGER__
           end
         end
 
-      # skip
-      when 'bv'
-        check_postmortem
-        require 'json'
-
-        h = Hash.new{|h, k| h[k] = []}
-        @bps.each_value{|bp|
-          if LineBreakpoint === bp
-            h[bp.path] << {lnum: bp.line}
-          end
-        }
-        if h.empty?
-          # TODO: clean?
-        else
-          open(".rdb_breakpoints.json", 'w'){|f| JSON.dump(h, f)}
-        end
-
-        vimsrc = File.join(__dir__, 'bp.vim')
-        system("vim -R -S #{vimsrc} #{@tc.location.path}")
-
-        if File.exist?(".rdb_breakpoints.json")
-          pp JSON.load(File.read(".rdb_breakpoints.json"))
-        end
-
-        return :retry
-
       # * `catch <Error>`
       #   * Set breakpoint on raising `<Error>`.
       # * `catch ... if: <expr>`
@@ -566,8 +553,8 @@ module DEBUGGER__
       #   * runs `<command>` before stopping.
       # * `catch ... do: <command>`
       #   * stops and run `<command>`, and continue.
-      # * `catch ... path: <path_regexp>`
-      #   * stops if the exception is raised from a path that matches <path_regexp>.
+      # * `catch ... path: <path>`
+      #   * stops if the exception is raised from a `<path>`. `<path>` can be a regexp with `/regexp/`.
       when 'catch'
         check_postmortem
 
@@ -588,8 +575,8 @@ module DEBUGGER__
       #   * runs `<command>` before stopping.
       # * `watch ... do: <command>`
       #   * stops and run `<command>`, and continue.
-      # * `watch ... path: <path_regexp>`
-      #   * stops if the triggering event's path matches <path_regexp>.
+      # * `watch ... path: <path>`
+      #   * stops if the path matches `<path>`. `<path>` can be a regexp with `/regexp/`.
       when 'wat', 'watch'
         check_postmortem
 
@@ -635,15 +622,15 @@ module DEBUGGER__
       when 'bt', 'backtrace'
         case arg
         when /\A(\d+)\z/
-          @tc << [:show, :backtrace, arg.to_i, nil]
+          request_tc [:show, :backtrace, arg.to_i, nil]
         when /\A\/(.*)\/\z/
           pattern = $1
-          @tc << [:show, :backtrace, nil, Regexp.compile(pattern)]
+          request_tc [:show, :backtrace, nil, Regexp.compile(pattern)]
         when /\A(\d+)\s+\/(.*)\/\z/
           max, pattern = $1, $2
-          @tc << [:show, :backtrace, max.to_i, Regexp.compile(pattern)]
+          request_tc [:show, :backtrace, max.to_i, Regexp.compile(pattern)]
         else
-          @tc << [:show, :backtrace, nil, nil]
+          request_tc [:show, :backtrace, nil, nil]
         end
 
       # * `l[ist]`
@@ -656,13 +643,13 @@ module DEBUGGER__
       when 'l', 'list'
         case arg ? arg.strip : nil
         when /\A(\d+)\z/
-          @tc << [:show, :list, {start_line: arg.to_i - 1}]
+          request_tc [:show, :list, {start_line: arg.to_i - 1}]
         when /\A-\z/
-          @tc << [:show, :list, {dir: -1}]
+          request_tc [:show, :list, {dir: -1}]
         when /\A(\d+)-(\d+)\z/
-          @tc << [:show, :list, {start_line: $1.to_i - 1, end_line: $2.to_i}]
+          request_tc [:show, :list, {start_line: $1.to_i - 1, end_line: $2.to_i}]
         when nil
-          @tc << [:show, :list]
+          request_tc [:show, :list]
         else
           @ui.puts "Can not handle list argument: #{arg}"
           return :retry
@@ -686,7 +673,7 @@ module DEBUGGER__
           return :retry
         end
 
-        @tc << [:show, :edit, arg]
+        request_tc [:show, :edit, arg]
 
       # * `i[nfo]`
       #    * Show information about current frame (local/instance variables and defined constants).
@@ -699,8 +686,8 @@ module DEBUGGER__
       #   * Show information about accessible constants except toplevel constants.
       # * `i[nfo] g[lobal[s]]`
       #   * Show information about global variables
-      # * `i[nfo] ... </pattern/>`
-      #   * Filter the output with `</pattern/>`.
+      # * `i[nfo] ... /regexp/`
+      #   * Filter the output with `/regexp/`.
       # * `i[nfo] th[read[s]]`
       #   * Show all threads (same as `th[read]`).
       when 'i', 'info'
@@ -713,15 +700,15 @@ module DEBUGGER__
 
         case sub
         when nil
-          @tc << [:show, :default, pat] # something useful
+          request_tc [:show, :default, pat] # something useful
         when 'l', /^locals?/
-          @tc << [:show, :locals, pat]
+          request_tc [:show, :locals, pat]
         when 'i', /^ivars?/i, /^instance[_ ]variables?/i
-          @tc << [:show, :ivars, pat]
+          request_tc [:show, :ivars, pat]
         when 'c', /^consts?/i, /^constants?/i
-          @tc << [:show, :consts, pat]
+          request_tc [:show, :consts, pat]
         when 'g', /^globals?/i, /^global[_ ]variables?/i
-          @tc << [:show, :globals, pat]
+          request_tc [:show, :globals, pat]
         when 'th', /threads?/
           thread_list
           return :retry
@@ -737,7 +724,7 @@ module DEBUGGER__
       #   * Show you available methods and instance variables of the given object.
       #   * If the object is a class/module, it also lists its constants.
       when 'outline', 'o', 'ls'
-        @tc << [:show, :outline, arg]
+        request_tc [:show, :outline, arg]
 
       # * `display`
       #   * Show display setting.
@@ -746,9 +733,9 @@ module DEBUGGER__
       when 'display'
         if arg && !arg.empty?
           @displays << arg
-          @tc << [:eval, :try_display, @displays]
+          request_tc [:eval, :try_display, @displays]
         else
-          @tc << [:eval, :display, @displays]
+          request_tc [:eval, :display, @displays]
         end
 
       # * `undisplay`
@@ -761,7 +748,7 @@ module DEBUGGER__
           if @displays[n = $1.to_i]
             @displays.delete_at n
           end
-          @tc << [:eval, :display, @displays]
+          request_tc [:eval, :display, @displays]
         when nil
           if ask "clear all?", 'N'
             @displays.clear
@@ -776,29 +763,29 @@ module DEBUGGER__
       # * `f[rame] <framenum>`
       #   * Specify a current frame. Evaluation are run on specified frame.
       when 'frame', 'f'
-        @tc << [:frame, :set, arg]
+        request_tc [:frame, :set, arg]
 
       # * `up`
       #   * Specify the upper frame.
       when 'up'
-        @tc << [:frame, :up]
+        request_tc [:frame, :up]
 
       # * `down`
       #   * Specify the lower frame.
       when 'down'
-        @tc << [:frame, :down]
+        request_tc [:frame, :down]
 
       ### Evaluate
 
       # * `p <expr>`
       #   * Evaluate like `p <expr>` on the current frame.
       when 'p'
-        @tc << [:eval, :p, arg.to_s]
+        request_tc [:eval, :p, arg.to_s]
 
       # * `pp <expr>`
       #   * Evaluate like `pp <expr>` on the current frame.
       when 'pp'
-        @tc << [:eval, :pp, arg.to_s]
+        request_tc [:eval, :pp, arg.to_s]
 
       # * `eval <expr>`
       #   * Evaluate `<expr>` on the current frame.
@@ -808,7 +795,7 @@ module DEBUGGER__
           @ui.puts "\nTo evaluate the variable `#{cmd}`, use `pp #{cmd}` instead."
           return :retry
         else
-          @tc << [:eval, :call, arg]
+          request_tc [:eval, :call, arg]
         end
 
       # * `irb`
@@ -818,7 +805,7 @@ module DEBUGGER__
           @ui.puts "not supported on the remote console."
           return :retry
         end
-        @tc << [:eval, :irb]
+        request_tc [:eval, :irb]
 
         # don't repeat irb command
         @repl_prev_line = nil
@@ -834,8 +821,8 @@ module DEBUGGER__
       #   * Add an exception tracer. It indicates raising exceptions.
       # * `trace object <expr>`
       #   * Add an object tracer. It indicates that an object by `<expr>` is passed as a parameter or a receiver on method call.
-      # * `trace ... </pattern/>`
-      #   * Indicates only matched events to `</pattern/>` (RegExp).
+      # * `trace ... /regexp/`
+      #   * Indicates only matched events to `/regexp/`.
       # * `trace ... into: <file>`
       #   * Save trace information into: `<file>`.
       # * `trace off <num>`
@@ -875,7 +862,7 @@ module DEBUGGER__
           return :retry
 
         when /\Aobject\s+(.+)/
-          @tc << [:trace, :object, $1.strip, {pattern: pattern, into: into}]
+          request_tc [:trace, :object, $1.strip, {pattern: pattern, into: into}]
 
         when /\Aoff\s+(\d+)\z/
           if t = @tracers.values[$1.to_i]
@@ -913,7 +900,7 @@ module DEBUGGER__
       when 'record'
         case arg
         when nil, 'on', 'off'
-          @tc << [:record, arg&.to_sym]
+          request_tc [:record, arg&.to_sym]
         else
           @ui.puts "unknown command: #{arg}"
           return :retry
@@ -978,7 +965,7 @@ module DEBUGGER__
       when 'open'
         case arg&.downcase
         when '', nil
-          repl_open_unix
+          repl_open
         when 'vscode'
           repl_open_vscode
         when /\A(.+):(\d+)\z/
@@ -1003,16 +990,12 @@ module DEBUGGER__
       # * `h[elp] <command>`
       #   * Show help for the given command.
       when 'h', 'help', '?'
-        if arg
-          show_help arg
-        else
-          @ui.puts DEBUGGER__.help
-        end
+        show_help arg
         return :retry
 
       ### END
       else
-        @tc << [:eval, :pp, line]
+        request_tc [:eval, :pp, line]
 =begin
         @repl_prev_line = nil
         @ui.puts "unknown command: #{line}"
@@ -1055,21 +1038,21 @@ module DEBUGGER__
       repl_open_setup
     end
 
-    def repl_open_unix
-      DEBUGGER__.open_unix nonstop: true
+    def repl_open
+      DEBUGGER__.open nonstop: true
       repl_open_setup
     end
 
     def repl_open_vscode
       CONFIG[:open_frontend] = 'vscode'
-      repl_open_unix
+      repl_open
     end
 
     def step_command type, arg
       case arg
       when nil, /\A\d+\z/
         if type == :in && @tc.recorder&.replaying?
-          @tc << [:step, type, arg&.to_i]
+          request_tc [:step, type, arg&.to_i]
         else
           leave_subsession [:step, type, arg&.to_i]
         end
@@ -1078,7 +1061,7 @@ module DEBUGGER__
           @ui.puts "only `step #{arg}` is supported."
           :retry
         else
-          @tc << [:step, arg.to_sym]
+          request_tc [:step, arg.to_sym]
         end
       else
         @ui.puts "Unknown option: #{arg}"
@@ -1088,11 +1071,18 @@ module DEBUGGER__
 
     def config_show key
       key = key.to_sym
-      if CONFIG_SET[key]
+      config_detail = CONFIG_SET[key]
+
+      if config_detail
         v = CONFIG[key]
-        kv = "#{key} = #{v.nil? ? '(default)' : v.inspect}"
-        desc = CONFIG_SET[key][1]
-        line = "%-30s \# %s" % [kv, desc]
+        kv = "#{key} = #{v.inspect}"
+        desc = config_detail[1]
+
+        if config_default = config_detail[3]
+          desc += " (default: #{config_default})"
+        end
+
+        line = "%-34s \# %s" % [kv, desc]
         if line.size > SESSION.width
           @ui.puts "\# #{desc}\n#{kv}"
         else
@@ -1142,7 +1132,7 @@ module DEBUGGER__
         config_set $1, $2, append: true
 
       when /\A\s*append\s+(\w+)\s+(.+)\z/
-        config_set $1, $2
+        config_set $1, $2, append: true
 
       when /\A(\w+)\z/
         config_show $1
@@ -1159,16 +1149,50 @@ module DEBUGGER__
       end
     end
 
-    def show_help arg
-      DEBUGGER__.helps.each{|cat, cs|
-        cs.each{|ws, desc|
-          if ws.include? arg
-            @ui.puts desc
-            return
+    def show_help arg = nil
+      instructions = (DEBUGGER__.commands.keys + DEBUGGER__.commands.values).uniq
+      print_instructions = proc do |desc|
+        desc.split("\n").each do |line|
+          next if line.start_with?(" ") # workaround for step back
+          formatted_line = line.gsub(/[\[\]\*]/, "").strip
+          instructions.each do |inst|
+            if formatted_line.start_with?("`#{inst}")
+              desc.sub!(line, colorize(line, [:CYAN, :BOLD]))
+            end
+          end
+        end
+        @ui.puts desc
+      end
+
+      print_category = proc do |cat|
+        @ui.puts "\n"
+        @ui.puts colorize("### #{cat}", [:GREEN, :BOLD])
+        @ui.puts "\n"
+      end
+
+      DEBUGGER__.helps.each { |cat, cs|
+        # categories
+        if arg.nil?
+          print_category.call(cat)
+        else
+          cs.each { |ws, _|
+            if ws.include?(arg)
+              print_category.call(cat)
+              break
+            end
+          }
+        end
+
+        # instructions
+        cs.each { |ws, desc|
+          if arg.nil? || ws.include?(arg)
+            print_instructions.call(desc.dup)
+            return if arg
           end
         }
       }
-      @ui.puts "not found: #{arg}"
+
+      @ui.puts "not found: #{arg}" if arg
     end
 
     def ask msg, default = 'Y'
@@ -1234,9 +1258,12 @@ module DEBUGGER__
       @repl_prev_line = nil
 
       if @bps.has_key? bp.key
-        unless bp.duplicable?
+        if bp.duplicable?
+          bp
+        else
           @ui.puts "duplicated breakpoint: #{bp}"
           bp.disable
+          nil
         end
       else
         @bps[bp.key] = bp
@@ -1272,14 +1299,20 @@ module DEBUGGER__
         end
       }
       expr.default_proc = nil
-      expr.transform_values{|v| v.join(' ')}
+      expr = expr.transform_values{|v| v.join(' ')}
+
+      if (path = expr[:path]) && path =~ /\A\/(.*)\/\z/
+        expr[:path] = Regexp.compile($1)
+      end
+
+      expr
     end
 
     def repl_add_breakpoint arg
       expr = parse_break arg.strip
       cond = expr[:if]
       cmd = ['break', expr[:pre], expr[:do]] if expr[:pre] || expr[:do]
-      path = Regexp.compile(expr[:path]) if expr[:path]
+      path = expr[:path]
 
       case expr[:sig]
       when /\A(\d+)\z/
@@ -1287,10 +1320,10 @@ module DEBUGGER__
       when /\A(.+)[:\s+](\d+)\z/
         add_line_breakpoint $1, $2.to_i, cond: cond, command: cmd
       when /\A(.+)([\.\#])(.+)\z/
-        @tc << [:breakpoint, :method, $1, $2, $3, cond, cmd, path]
+        request_tc [:breakpoint, :method, $1, $2, $3, cond, cmd, path]
         return :noretry
       when nil
-        add_check_breakpoint cond, path
+        add_check_breakpoint cond, path, cmd
       else
         @ui.puts "Unknown breakpoint format: #{arg}"
         @ui.puts
@@ -1302,7 +1335,7 @@ module DEBUGGER__
       expr = parse_break arg.strip
       cond = expr[:if]
       cmd = ['catch', expr[:pre], expr[:do]] if expr[:pre] || expr[:do]
-      path = Regexp.compile(expr[:path]) if expr[:path]
+      path = expr[:path]
 
       bp = CatchBreakpoint.new(expr[:sig], cond: cond, command: cmd, path: path)
       add_bp bp
@@ -1314,16 +1347,16 @@ module DEBUGGER__
       cmd = ['watch', expr[:pre], expr[:do]] if expr[:pre] || expr[:do]
       path = Regexp.compile(expr[:path]) if expr[:path]
 
-      @tc << [:breakpoint, :watch, expr[:sig], cond, cmd, path]
+      request_tc [:breakpoint, :watch, expr[:sig], cond, cmd, path]
     end
 
-    def add_catch_breakpoint pat
-      bp = CatchBreakpoint.new(pat)
+    def add_catch_breakpoint pat, cond: nil
+      bp = CatchBreakpoint.new(pat, cond: cond)
       add_bp bp
     end
 
-    def add_check_breakpoint expr, path
-      bp = CheckBreakpoint.new(expr, path)
+    def add_check_breakpoint cond, path, command
+      bp = CheckBreakpoint.new(cond: cond, path: path, command: command)
       add_bp bp
     end
 
@@ -1336,6 +1369,34 @@ module DEBUGGER__
       @ui.puts e.message
     end
 
+    def clear_breakpoints(&condition)
+      @bps.delete_if do |k, bp|
+        if condition.call(k, bp)
+          bp.delete
+          true
+        end
+      end
+    end
+
+    def clear_line_breakpoints path
+      path = resolve_path(path)
+      clear_breakpoints do |k, bp|
+        bp.is_a?(LineBreakpoint) && DEBUGGER__.compare_path(k.first, path)
+      end
+    rescue Errno::ENOENT
+      # just ignore
+    end
+
+    def clear_catch_breakpoints *exception_names
+      clear_breakpoints do |k, bp|
+        bp.is_a?(CatchBreakpoint) && exception_names.include?(k[1])
+      end
+    end
+
+    def clear_all_breakpoints
+      clear_breakpoints{true}
+    end
+
     def add_iseq_breakpoint iseq, **kw
       bp = ISeqBreakpoint.new(iseq, [:line], **kw)
       add_bp bp
@@ -1503,42 +1564,66 @@ module DEBUGGER__
     end
 
     private def enter_subsession
-      raise "already in subsession" if @subsession
-      @subsession = true
-      stop_all_threads
-      @process_group.lock
-      DEBUGGER__.info "enter_subsession"
+      if !@subsession_stack.empty?
+        DEBUGGER__.info "Enter subsession (nested #{@subsession_stack.size})"
+      else
+        DEBUGGER__.info "Enter subsession"
+        stop_all_threads
+        @process_group.lock
+      end
+
+      @subsession_stack << true
     end
 
     private def leave_subsession type
-      DEBUGGER__.info "leave_subsession"
-      @process_group.unlock
-      restart_all_threads
-      @tc << type if type
+      raise '[BUG] leave_subsession: not entered' if @subsession_stack.empty?
+      @subsession_stack.pop
+
+      if @subsession_stack.empty?
+        DEBUGGER__.info "Leave subsession"
+        @process_group.unlock
+        restart_all_threads
+      else
+        DEBUGGER__.info "Leave subsession (nested #{@subsession_stack.size})"
+      end
+
+      request_tc type if type
       @tc = nil
-      @subsession = false
     rescue Exception => e
-      STDERR.puts [e, e.backtrace].inspect
+      STDERR.puts PP.pp([e, e.backtrace], ''.dup)
       raise
     end
 
     def in_subsession?
-      @subsession
+      !@subsession_stack.empty?
     end
 
     ## event
 
     def on_load iseq, src
       DEBUGGER__.info "Load #{iseq.absolute_path || iseq.path}"
-      @sr.add iseq, src
+
+      file_path, reloaded = @sr.add(iseq, src)
+      @ui.event :load, file_path, reloaded
 
       pending_line_breakpoints = @bps.find_all do |key, bp|
         LineBreakpoint === bp && !bp.iseq
       end
 
       pending_line_breakpoints.each do |_key, bp|
-        if bp.path == (iseq.absolute_path || iseq.path)
-          bp.try_activate
+        if DEBUGGER__.compare_path(bp.path, (iseq.absolute_path || iseq.path))
+          bp.try_activate iseq
+        end
+      end
+
+      if reloaded
+        @bps.find_all do |key, bp|
+          LineBreakpoint === bp && DEBUGGER__.compare_path(bp.path, file_path)
+        end.each do |_key, bp|
+          @bps.delete bp.key # to allow duplicate
+          if nbp = LineBreakpoint.copy(bp, iseq)
+            add_bp nbp
+          end
         end
       end
     end
@@ -1563,9 +1648,10 @@ module DEBUGGER__
 
     def method_added tp
       b = tp.binding
+
       if var_name = b.local_variables.first
         mid = b.local_variable_get(var_name)
-        unresolved = false
+        resolved = true
 
         @bps.each{|k, bp|
           case bp
@@ -1576,15 +1662,57 @@ module DEBUGGER__
               end
             end
 
-            unresolved = true unless bp.enabled?
+            resolved = false if !bp.enabled?
           end
         }
-        unless unresolved
-          METHOD_ADDED_TRACKER.disable
+
+        if resolved
+          Session.deactivate_method_added_trackers
+        end
+
+        case mid
+        when :method_added, :singleton_method_added
+          Session.create_method_added_tracker(tp.self, mid)
+          Session.activate_method_added_trackers unless resolved
         end
       end
     end
 
+    class ::Module
+      undef method_added
+      def method_added mid; end
+    end
+
+    class ::BasicObject
+      undef singleton_method_added
+      def singleton_method_added mid; end
+    end
+
+    def self.create_method_added_tracker mod, method_added_id, method_accessor = :method
+      m = mod.__send__(method_accessor, method_added_id)
+      METHOD_ADDED_TRACKERS[m] = TracePoint.new(:call) do |tp|
+        SESSION.method_added tp
+      end
+    end
+
+    def self.activate_method_added_trackers
+      METHOD_ADDED_TRACKERS.each do |m, tp|
+        tp.enable(target: m) unless tp.enabled?
+      rescue ArgumentError
+        DEBUGGER__.warn "Methods defined under #{m.owner} can not track by the debugger."
+      end
+    end
+
+    def self.deactivate_method_added_trackers
+      METHOD_ADDED_TRACKERS.each do |m, tp|
+        tp.disable if tp.enabled?
+      end
+    end
+
+    METHOD_ADDED_TRACKERS = Hash.new
+    create_method_added_tracker Module, :method_added, :instance_method
+    create_method_added_tracker BasicObject, :singleton_method_added, :instance_method
+
     def width
       @ui.width
     end
@@ -1765,9 +1893,11 @@ module DEBUGGER__
 
     def after_fork child: true
       if child || !@lock_file
-        @m = Mutex.new
-        @lock_level = 0
-        @lock_file = open(@lock_tempfile.path, 'w')
+        @m = Mutex.new unless @m
+        @m.synchronize do
+          @lock_level = 0
+          @lock_file = open(@lock_tempfile.path, 'w')
+        end
       end
     end
 
@@ -1860,6 +1990,9 @@ module DEBUGGER__
         puts "\nStop by #{args.first}"
       end
     end
+
+    def flush
+    end
   end
 
   # manual configuration methods
@@ -1876,7 +2009,7 @@ module DEBUGGER__
   # nil for -r
   def self.require_location
     locs = caller_locations
-    dir_prefix = /#{__dir__}/
+    dir_prefix = /#{Regexp.escape(__dir__)}/
 
     locs.each do |loc|
       case loc.absolute_path
@@ -1896,7 +2029,7 @@ module DEBUGGER__
 
     unless defined? SESSION
       require_relative 'local'
-      initialize_session UI_LocalConsole.new
+      initialize_session{ UI_LocalConsole.new }
     end
 
     setup_initial_suspend unless nonstop
@@ -1904,8 +2037,9 @@ module DEBUGGER__
 
   def self.open host: nil, port: CONFIG[:port], sock_path: nil, sock_dir: nil, nonstop: false, **kw
     CONFIG.set_config(**kw)
+    require_relative 'server'
 
-    if port || CONFIG[:open_frontend] == 'chrome'
+    if port || CONFIG[:open_frontend] == 'chrome' || (!::Addrinfo.respond_to?(:unix))
       open_tcp host: host, port: (port || 0), nonstop: nonstop
     else
       open_unix sock_path: sock_path, sock_dir: sock_dir, nonstop: nonstop
@@ -1919,7 +2053,7 @@ module DEBUGGER__
     if defined? SESSION
       SESSION.reset_ui UI_TcpServer.new(host: host, port: port)
     else
-      initialize_session UI_TcpServer.new(host: host, port: port)
+      initialize_session{ UI_TcpServer.new(host: host, port: port) }
     end
 
     setup_initial_suspend unless nonstop
@@ -1932,7 +2066,7 @@ module DEBUGGER__
     if defined? SESSION
       SESSION.reset_ui UI_UnixDomainServer.new(sock_dir: sock_dir, sock_path: sock_path)
     else
-      initialize_session UI_UnixDomainServer.new(sock_dir: sock_dir, sock_path: sock_path)
+      initialize_session{ UI_UnixDomainServer.new(sock_dir: sock_dir, sock_path: sock_path) }
     end
 
     setup_initial_suspend unless nonstop
@@ -1959,9 +2093,10 @@ module DEBUGGER__
   end
 
   class << self
-    define_method :initialize_session do |ui|
+    define_method :initialize_session do |&init_ui|
       DEBUGGER__.info "Session start (pid: #{Process.pid})"
-      ::DEBUGGER__.const_set(:SESSION, Session.new(ui))
+      ::DEBUGGER__.const_set(:SESSION, Session.new)
+      SESSION.activate init_ui.call
       load_rc
     end
   end
@@ -1993,34 +2128,53 @@ module DEBUGGER__
     end
   end
 
-  class ::Module
-    undef method_added
-    def method_added mid; end
-    def singleton_method_added mid; end
-  end
-
-  def self.method_added tp
-    begin
-      SESSION.method_added tp
-    rescue Exception => e
-      p e
-    end
-  end
-
-  METHOD_ADDED_TRACKER = self.create_method_added_tracker
+  # Inspector
 
   SHORT_INSPECT_LENGTH = 40
 
-  def self.safe_inspect obj, max_length: SHORT_INSPECT_LENGTH, short: false
-    str = obj.inspect
+  class LimitedPP
+    def self.pp(obj, max=80)
+      out = self.new(max)
+      catch out do
+        PP.singleline_pp(obj, out)
+      end
+      out.buf
+    end
 
-    if short && str.length > max_length
-      str[0...max_length] + '...'
+    attr_reader :buf
+
+    def initialize max
+      @max = max
+      @cnt = 0
+      @buf = String.new
+    end
+
+    def <<(other)
+      @buf << other
+
+      if @buf.size >= @max
+        @buf = @buf[0..@max] + '...'
+        throw self
+      end
+    end
+  end
+
+  def self.safe_inspect obj, max_length: SHORT_INSPECT_LENGTH, short: false
+    if short
+      LimitedPP.pp(obj, max_length)
     else
-      str
+      obj.inspect
+    end
+  rescue NoMethodError => e
+    klass, oid = M_CLASS.bind_call(obj), M_OBJECT_ID.bind_call(obj)
+    if obj == (r = e.receiver)
+      "<\##{klass.name}#{oid} does not have \#inspect>"
+    else
+      rklass, roid = M_CLASS.bind_call(r), M_OBJECT_ID.bind_call(r)
+      "<\##{klass.name}:#{roid} contains <\##{rklass}:#{roid} and it does not have #inspect>"
     end
   rescue Exception => e
-    str = "<#inspect raises #{e.inspect}>"
+    "<#inspect raises #{e.inspect}>"
   end
 
   def self.warn msg
@@ -2031,18 +2185,27 @@ module DEBUGGER__
     log :INFO, msg
   end
 
-  def self.log level, msg
-    @logfile = STDERR unless defined? @logfile
-
+  def self.check_loglevel level
     lv = LOG_LEVELS[level]
-    config_lv = LOG_LEVELS[CONFIG[:log_level] || :WARN]
+    config_lv = LOG_LEVELS[CONFIG[:log_level]]
+    lv <= config_lv
+  end
 
-    if defined? SESSION
-      pi = SESSION.process_info
-      process_info = pi ? "[#{pi}]" : nil
+  def self.debug(&b)
+    if check_loglevel :DEBUG
+      log :DEBUG, b.call
     end
+  end
+
+  def self.log level, msg
+    if check_loglevel level
+      @logfile = STDERR unless defined? @logfile
+
+      if defined? SESSION
+        pi = SESSION.process_info
+        process_info = pi ? "[#{pi}]" : nil
+      end
 
-    if lv <= config_lv
       if level == :WARN
         # :WARN on debugger is general information
         @logfile.puts "DEBUGGER#{process_info}: #{msg}"
@@ -2062,17 +2225,69 @@ module DEBUGGER__
     yield
   end
 
-  module ForkInterceptor
-    def fork(&given_block)
-      return super unless defined?(SESSION) && SESSION.active?
+  if File.identical?(__FILE__.upcase, __FILE__.downcase)
+    # For case insensitive file system (like Windows)
+    # Note that this check is not enough because case sensitive/insensitive is
+    # depend on the file system. So this check is only roughly estimation.
 
-      unless fork_mode = CONFIG[:fork_mode]
-        if CONFIG[:parent_on_fork]
-          fork_mode = :parent
-        else
-          fork_mode = :both
+    def self.compare_path(a, b)
+      a&.downcase == b&.downcase
+    end
+  else
+    def self.compare_path(a, b)
+      a == b
+    end
+  end
+
+  module ForkInterceptor
+    if Process.respond_to? :_fork
+      def _fork
+        return super unless defined?(SESSION) && SESSION.active?
+
+        parent_hook, child_hook = __fork_setup_for_debugger
+
+        super.tap do |pid|
+          if pid != 0
+            # after fork: parent
+            parent_hook.call pid
+          else
+            # after fork: child
+            child_hook.call
+          end
         end
       end
+    else
+      def fork(&given_block)
+        return super unless defined?(SESSION) && SESSION.active?
+        parent_hook, child_hook = __fork_setup_for_debugger
+
+        if given_block
+          new_block = proc {
+            # after fork: child
+            child_hook.call
+            given_block.call
+          }
+          super(&new_block).tap{|pid| parent_hook.call(pid)}
+        else
+          super.tap do |pid|
+            if pid
+              # after fork: parent
+              parent_hook.call pid
+            else
+              # after fork: child
+              child_hook.call
+            end
+          end
+        end
+      end
+    end
+
+    private def __fork_setup_for_debugger
+      fork_mode = CONFIG[:fork_mode]
+
+      if fork_mode == :both && CONFIG[:parent_on_fork]
+        fork_mode = :parent
+      end
 
       parent_pid = Process.pid
 
@@ -2112,26 +2327,7 @@ module DEBUGGER__
         }
       end
 
-      if given_block
-        new_block = proc {
-          # after fork: child
-          child_hook.call
-          given_block.call
-        }
-        pid = super(&new_block)
-        parent_hook.call(pid)
-        pid
-      else
-        if pid = super
-          # after fork: parent
-          parent_hook.call pid
-        else
-          # after fork: child
-          child_hook.call
-        end
-
-        pid
-      end
+      return parent_hook, child_hook
     end
   end
 
@@ -2148,28 +2344,46 @@ module DEBUGGER__
     end
   end
 
-  if RUBY_VERSION >= '3.0.0'
+  if Process.respond_to? :_fork
+    module ::Process
+      class << self
+        prepend ForkInterceptor
+      end
+    end
+
+    # trap
     module ::Kernel
-      prepend ForkInterceptor
       prepend TrapInterceptor
     end
+    module ::Signal
+      class << self
+        prepend TrapInterceptor
+      end
+    end
   else
-    class ::Object
-      include ForkInterceptor
-      include TrapInterceptor
+    if RUBY_VERSION >= '3.0.0'
+      module ::Kernel
+        prepend ForkInterceptor
+        prepend TrapInterceptor
+      end
+    else
+      class ::Object
+        include ForkInterceptor
+        include TrapInterceptor
+      end
     end
-  end
 
-  module ::Kernel
-    class << self
-      prepend ForkInterceptor
-      prepend TrapInterceptor
+    module ::Kernel
+      class << self
+        prepend ForkInterceptor
+        prepend TrapInterceptor
+      end
     end
-  end
 
-  module ::Process
-    class << self
-      prepend ForkInterceptor
+    module ::Process
+      class << self
+        prepend ForkInterceptor
+      end
     end
   end
 
@@ -2199,3 +2413,12 @@ class Binding
   alias break debugger
   alias b debugger
 end
+
+# for Ruby 2.6 compatibility
+unless method(:p).unbind.respond_to? :bind_call
+  class UnboundMethod
+    def bind_call(obj, *args)
+      self.bind(obj).call(*args)
+    end
+  end
+end
